function [SDR_out match_matrix] = SF_BSS_eval(sources_cell, separated_cell,Fs,nb,outname,eval_name)

% SF_BSS_eval(sources, separated, Fs,nb,outname,eval_name)
%
% Calculates Signal to Distortion, to Interference and to Artifacts
% Ratio using bss_eval toolbox available in http://bass-db.gforge.inria.fr/bss_eval/
%
% INPUT
% sources: a matrix containing all original sources, each in one line
% separated: a matrix containing the separated signals, each in one line
% Fs: Sample Rate
% nb: number of bits of the signal representation
% outname: 
% eval_name: evaluation results file. Choose 0 to print on screen
% Author: Alan F. Tygel
% Last Modified: 01/02/2009


addpath('../bss_eval/');

D = length(sources_cell);
sources = zeros(D,length(sources_cell{1}.signal));
separated = zeros(D,length(separated_cell{1}.signal));

for d = 1:D
	sources(d,:) = sources_cell{d}.signal;
	separated(d,:) = separated_cell{d}.signal;
end


if(eval_name == 0)
fid = 1;
else
fid = fopen(eval_name,'a');
end

%N_sources = size(sources,1);
D = size(sources,1);

SIR = zeros(1,D);
SDR = zeros(1,D);
SAR = zeros(1,D);

SDR_out = zeros(1,D);

match_matrix = zeros(D,2);

for sour = 1:D
		for sep = 1:D
%
%          [x y] = SF_aligner(separated(sep,:),sources(sour,:),Fs,nb);
					x = separated(sep,:);
%          size(separated)
%          size(x)
%          size(sources)

				if (length(x) < size(sources,2))
					x = [x  zeros(1,size(sources,2)-length(x))];
				end

        [s_target e_interf e_artif] = bss_decomp_gain(x,sour,sources); 
        [SDR(sep),SIR(sep),SAR(sep)] = bss_crit(s_target,e_interf,e_artif);
        
    end
    [xx corr_sep] = max(SDR);

    %in columns: outname | Separated | Source | SDR | SIR | SAR
    fprintf(fid,'%s , %d , %d , %1.3d , %1.3d , %1.3d \n',outname, sour ,corr_sep,SDR(corr_sep),SIR(corr_sep),SAR(corr_sep));
    
    SDR_out(sour) = SDR(corr_sep);
    
		match_matrix(sour,:) = [sour corr_sep];  


end

if(eval_name ~= 0)
fclose(fid);
end



end