function [X FFT_T hopsize PH] = SF_fast_cqt(x,Fs,fMin,fMax)

% [X FFT_T hopsize PH] = SF_fast_cqt(x,Fs,fMin,fMax)
%
% Performs the fast implementation of the Constant Q Transform
% 
% INPUT
% x: signal in time domais
% Fs: Sample Rate
% fMin: minimum freq. in Hz to be considered
% fMax: maximum freq. in Hz to be considered
%
% OUTPUT
% X: Linear spectrogram (with phase)
% FFT_T: fourier transform of the CQT matrix
% hopsize: hopsize
% PH: phase of the spectrogram
%
%
% Author: Alan F. Tygel
% Last Modified: 30/01/2009

%fast CQT
%

Q = 34; %quarter tone per bin
overlap = 0.5; %window overlap
kMax = round(24*log2(fMax/fMin)); %total number of CQT bins

k = 1:kMax;
freq = ((2.^(1/24)).^(k)).*fMin;
N = round(Fs./freq.*Q);
for	k = 1:kMax
	if mod(N(k),2) == 0
		N(k) = N(k) + 1;
	end
	W{k} = hanning(N(k));
end

hopsize = round(min(N)*(1-overlap));
maxWSize = max(N);
X = zeros(kMax,round((length(x)+maxWSize)/hopsize));

%CQT martix:
T = zeros(maxWSize, kMax);

centro = (maxWSize+1)/2;
for k = 1:kMax
  for n1 = centro-(N(k)-1)/2:centro+(N(k)-1)/2;
    n = n1 - (centro-(N(k)-1)/2) +1;
    T(n1,k) = (1/N(k)) * W{k}(n) * exp(-j*2*pi*Q*n/N(k));
  end
end

FFT_T = fft(T,[],1);

X2 = zeros(maxWSize,round((length(x)+maxWSize)/hopsize));
x = [zeros(maxWSize,1); x ;zeros(maxWSize,1)];
frames = 1;
for n_centro = (maxWSize + 1) : hopsize : length(x)-maxWSize  %frames
 	n = n_centro-((maxWSize-1)/2) : n_centro + ((maxWSize-1)/2); 
	FFT_X = fft(x(n)');
	X2(:,frames) = FFT_X;% * FFT_T;
	frames = frames+1;	
end

%FFT_T = FFT_T.';


X = FFT_T.'*X2;

figure
imagesc(abs(X))
figure
imagesc(abs(X2))
figure
imagesc(abs(FFT_T*X))
figure
imagesc(abs(pinv(FFT_T.')*X))


PH = zeros(size(X));
for kk=1:size(X,2)
    PH(:,kk) = phase(X(:,kk));
end


end