function Y = SF_lin_reconstruct(W,H,b,M)

% Y = SF_lin_reconstruct(W,H,b)
%
% Reconstruction of the linear-spectrogram given matrices W and H,
% and optionally M. Use M=0 to disconsider Spectral Adaptation.
%
%
% INPUT
% W: W matrix containing spectral description
% H: H matrix, containing temporal description
% b: log-frequency resolution (12 for semitone, 24 for quartertone,...)
% M: M matrix, spectral adaptation matriz (OPTIONAL)
%
% OUTPUT
% Y: Cell containing each sources log-spectrogram
%
% Author: Alan F. Tygel
% Last Modified: 28/01/2009

D = size(W,2);
TAU = size(W,3);
PHI = size(H,3);

Y = cell(D);

if(M == 0)
	M = ones(size(W,1),TAU,PHI,D);
end


for d=1:D
    Lambda = zeros(size(W,1),size(H,2));
    for tau = 1:TAU
        for phi = 1:PHI
            Lambda = Lambda + (SF_v_linear_shift(W(:,d,tau),phi-1,b).*M(:,tau,phi,d))*SF_h_shift(H(d,:,phi),tau-1);
        end
    end
    Y{d} = Lambda;
end

end