function [W H KLDiv] = SF_lnmf2d(X,TAU,PHI,D,b,max_it,delta,W_ini,H_ini)

% [W H] = SF_lnmf2d(X, TAU, PHI, D, b, max_it, delta)
% Performs Linear Non-negative matrix factor 2-D Deconvolution on
% linear-spectrogram X using Kullback Leibler Divergence as target function.
%
% INPUT
% X: log-spectrogram
% TAU: number of frames in each factor
% PHI: possible frequency translations
% D: number of instruments
% b: log-frequency resolution (12 for semitone, 24 for quartertone,...)
% max_it: maximum number of iterations
% delta: difference between successives values of target function under which
% convergence will be considered achieved
% W_ini: inital value for matrix W
% H_ini: inital value for matrix H
%
%
% OUTPUT
% W: W matrix at the end of optimization
% H: H matrix at the end of optimization
%
%
% Author: Alan F. Tygel
% Last Modified: 28/01/2009

global disp_targ disp_plot

X = abs(X);
X = X(1:end/2+1,:);
X = X./max(max(X));

I = size(X,1);
J = size(X,2);

%max_it = 1000;
KLDiv = zeros(1,max_it);
%KLDiv_min = 10e10;
KLDiv(1) = 10e10;
KLDiv(2) = 10e10;
%Delta = 1e-9;

thres = 1e-5;

%X(find(X < thres)) = thres;

V = X + thres;
V = V./max(max(V));

if (W_ini == 0)
	W = rand(I,D,TAU);
else
	W = W_ini;
end

if (H_ini == 0)
	H = rand(D,J,PHI);
else
	H = H_ini;
end

it = 2;


while (1==1)
%tic
it = it+1;

if(it == max_it)
    fprintf('Maximum iterations reached!\n');
    break;
end


Lambda = zeros(I,J);
for tau = 1:TAU
    for phi = 1:PHI
        Lambda = Lambda + SF_v_linear_shift(W(:,:,tau),phi-1,b)*SF_h_shift(H(:,:,phi),tau-1);
    end
end
%Lambda(find(Lambda < thres)) = V(find(Lambda < thres));
%Lambda = Lambda/max(max(Lambda));
%div (find(Lambda < 10e-5)) = 0;
div = V./Lambda;

%%%%%%%%%%%%%%%%%%%K.L.Divergence

KLDiv(it) = sum(sum(V(:,:).*log(div(:,:)) - V(:,:) + Lambda(:,:)));

if(strcmp(disp_targ,'on'))
	fprintf('Div: %d\t It: %d\n',KLDiv(it),it);
	if(isnan(KLDiv(it)))
		error('Error: KLDiv is not a number.');
	end
end

if(abs(KLDiv(it)-KLDiv(it-2)) < delta)
    fprintf('Converged at iteration %d\n',it);
    break;
end

if(KLDiv(it)-KLDiv(it-2) > 0)  
    fprintf('Objective Function growing at it. %d. Stopping optimization.\n',it);
    break;
end


% if(KLDiv(it) < KLDiv_min)
% 	W_min = W;
% 	H_min = H;
% 	KLDiv_min = KLDiv(it);
% 	it_min = it;
% end


%%%%%%%%%MATRIX UPDATES%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(mod(it,2)==0)
  if(strcmp(disp_targ,'on')), fprintf('Update W \n'), end;

  %%%%W UPADTE
  for tau = 1:TAU
    AUX1 = zeros(I,D);
    for phi = 1:PHI
      AUX1 = AUX1 + SF_v_linear_shift(div,-(phi-1),b)*SF_h_shift(H(:,:,phi),tau-1)';
    end
    AUX2 = ones(I,J)*SF_h_shift(sum(H,3),tau-1)';    
    W(:,:,tau) = W(:,:,tau) .* (AUX1./AUX2);
  end




else
  %%%%H UPDATE
  if(strcmp(disp_targ,'on')), fprintf('Update H \n'), end; 
  for phi = 1:PHI
    AUX1 = zeros(D,J);
    for tau = 1:TAU
      AUX1 = AUX1 + SF_v_linear_shift(W(:,:,tau),phi-1,b)'*SF_h_shift(div,-(tau-1));
    end
    AUX2 = SF_v_linear_shift(sum(W,3),phi-1,b)'*ones(I,J);    
    H(:,:,phi) = H(:,:,phi) .* (AUX1./AUX2);
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end %if(mod(it,2)==0)

%%%%%%%%%%%%Matrixes Display %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(strcmp(disp_plot,'on'))
  SF_disp_plot_f(W,H,Lambda);
end


end %while

end %function
