function [W H MSEDiv] = SF_lnmf2d_MSE(X,TAU,PHI,D,b,max_it,delta,W_ini,H_ini)

% [W H] = SF_lnmf2d_MSE(X, TAU, PHI, D, b, max_it, delta)
% Performs Linear Non-negative matrix factor 2-D Deconvolution on
% linear-spectrogram X using MSE as target function.
%
% INPUT
% X: log-spectrogram
% TAU: number of frames in each factor
% PHI: possible frequency translations
% D: number of instruments
% b: log-frequency resolution (12 for semitone, 24 for quartertone,...)
% max_it: maximum number of iterations
% delta: difference between successives values of target function under which
% convergence will be considered achieved
% W_ini: inital value for matrix W
% H_ini: inital value for matrix H
%
%
% OUTPUT
% W: W matrix at the end of optimization
% H: H matrix at the end of optimization
%
%
% Author: Alan F. Tygel
% Last Modified: 28/01/2009

global disp_targ disp_plot

X = abs(X);
X = X(1:end/2+1,:);
X = X./max(max(X));

I = size(X,1);
J = size(X,2);

MSEDiv = zeros(1,max_it);
% MSEDiv_min = 10e10;
MSEDiv(1) = 10e10;
MSEDiv(2) = 10e10;

thres = 1e-5;

V = X +thres;
%V(find(V < thres)) = 0;

V = V./max(max(V));

if (W_ini == 0)
	W = rand(I,D,TAU);
else
	W = W_ini;
end


if (H_ini == 0)
	H = rand(D,J,PHI);
else
	H = H_ini;
end

it = 2;


while (1==1)
%tic
it = it+1;

if(it == max_it)
    fprintf('Maximum iterations reached!\n');
    break;
end

Lambda = zeros(I,J);
for tau = 1:TAU
    for phi = 1:PHI
        Lambda = Lambda + SF_v_linear_shift(W(:,:,tau),phi-1,b)*SF_h_shift(H(:,:,phi),tau-1);
    end
end
%Lambda(find(Lambda < thres)) = V(find(Lambda < thres));
%  Lambda = Lambda/max(max(Lambda));


%%%%%%%%%%%%%%%%%%%M.S.E
MSEDiv(it) = sum(sum((V(:,:) - Lambda(:,:)).^2));

if(strcmp(disp_targ,'on'))
	fprintf('MSE: %d\t It: %d\n',MSEDiv(it),it);
	if(isnan(MSEDiv(it)))
		error('Error: MSEDiv is not a number.')
	end
end;

if(abs(MSEDiv(it)-MSEDiv(it-2)) < delta)
    fprintf('Convergiu! Iteracao %d\n',it);
    break;
end


%  if(MSEDiv(it)-MSEDiv(it-2) > 0)
%      fprintf('Objective Function growing at it. %d. Stopping optimization.\n',it);
%      break;
%  end

% if(MSEDiv(it) < MSEDiv_min)
% 	W_min = W;
% 	H_min = H;
% 	MSEDiv_min = MSEDiv(it);
% 	it_min = it;
% end  

	lambda = 1;
%  	eps = 1e-10;

%%%%%%%%%MATRIX UPDATES%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(mod(it,2)==0)
  if(strcmp(disp_targ,'on')), fprintf('Update W \n'), end;
  
  %%%%W UPADTE

	AUX1 = zeros(I,D,TAU);
	AUX2 = zeros(I,D,TAU);

  for tau = 1:TAU

    for phi = 1:PHI
      AUX1(:,:,tau) = AUX1(:,:,tau) + SF_v_linear_shift(V,-(phi-1),b)*SF_h_shift(H(:,:,phi),tau-1)';
      AUX2(:,:,tau) = AUX2(:,:,tau) + SF_v_linear_shift(Lambda,-(phi-1),b)*SF_h_shift(H(:,:,phi),tau-1)';
    end

  end

%			tx = sum(sum(AUX2.*W,1),3);
%   	ty = sum(sum(AUX1.*W,1),3);
%   	AUX1 = AUX1 + repmat(tx,[size(W,1),1,size(W,3)]).*W;
%   	AUX2 = AUX2 + repmat(ty,[size(W,1),1,size(W,3)]).*W;

%  	W = normalizeW(W .* (AUX1./(AUX2+eps)));
	W = W .* (AUX1./(AUX2+eps));


%  	norm_W = 0;
%  	for tau = 1:TAU
%  		norm_W = norm_W + norm(W(:,:,tau));
%  	end
%  	W = W./norm_W;

else

  %%%%H UPDATE
  if(strcmp(disp_targ,'on')), fprintf('Update H \n'), end;

	AUX1 = zeros(D,J,PHI);
	AUX2 = zeros(D,J,PHI);

  for phi = 1:PHI
    for tau = 1:TAU
      AUX1(:,:,phi) = AUX1(:,:,phi) + SF_v_linear_shift(W(:,:,tau),phi-1,b)'*SF_h_shift(V,-(tau-1));
      AUX2(:,:,phi) = AUX2(:,:,phi) + SF_v_linear_shift(W(:,:,tau),phi-1,b)'*SF_h_shift(Lambda,-(tau-1));
			
    end

  end

%    H = normalizeH(H .* (AUX1./(AUX2+lambda+eps)));
%    H = H .* (AUX1./(AUX2+lambda+eps));
		H = H .* (AUX1./(AUX2+eps));

end %if(mod(it,2)==0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%Matrixes Display %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(strcmp(disp_plot,'on'))
  SF_disp_plot_f(W,H,Lambda);
end



end %while

MSEDiv  = MSEDiv(1:find(MSEDiv==0,1)-1);

end %function


function W = normalizeW(W)
Q(1,:,1) = sqrt(sum(sum(W.^2,1),3));
W = W./repmat(Q+eps,[size(W,1),1,size(W,3)]);
end

function H = normalizeH(H)
Q(:,1,1) = sqrt(sum(sum(H.^2,2),3));
H = H./repmat(Q+eps,[1,size(H,2),size(H,3)]);
end