function [W H] = SF_nmf2d_MSE2(X, TAU, PHI, D, b, max_it, delta,W_ini,H_ini,sparse_factor)
						
% [W H] = SF_nmf2d_MSE(X, TAU, PHI, D, max_it, delta)
% Performs Non-negative matrix factor 2-D Deconvolution on log-spectrogram
% X using Mean-Square Error as target function.
%
% INPUT
% X: log-spectrogram
% TAU: number of frames in each factor
% PHI: possible frequency translations
% D: number of instruments
% max_it: maximum number of iterations
% delta: difference between successives values of target function under which
% convergence will be considered achieved
% W_ini: inital value for matrix W
% H_ini: inital value for matrix H
% lambda: sparsness factor
%
% OUTPUT
% W: W matrix at the end of optimization
% H: H matrix at the end of optimization
%
%
% Author: Alan F. Tygel
% Last Modified: 14/09/2009

global disp_targ disp_plot

V = abs(X) + 10e-9;
V = V./max(max(V));

I =size(X,1);
J =size(X,2);

nuW  = 1;
nuH  = 1;


%%%%%%%%INITIALIZE W AND H RANDOMLY
if (W_ini == 0), W = rand(I,D,TAU); else,  W = W_ini; end

if (H_ini == 0), H = rand(D,J,PHI);else , H = H_ini; end

new_W = zeros(I,D,TAU);
new_H = zeros(D,J,PHI);
W = normalizeW(W);

while (1==1)

	Lambda = lin_reconstruct(W,H,b);
	cost = calc_cost(V,Lambda,sparse_factor,H);

	while (1==1)
		%%%%%%%%%UPDATE W
		dW = zeros(I,D,TAU);
		
		for tau = 1:TAU
			for phi = 1:PHI
				dW(:,:,tau) = dW(:,:,tau) + SF_v_linear_shift(V-Lambda,-(phi-1),b)*SF_h_shift(H(:,:,phi),tau-1)';
			end
			new_W(:,:,tau) = W(:,:,tau) + nuW*dW(:,:,tau);
		end
		
		new_W(new_W <= 0) = eps;
	
		%%%%%%%%%NORMALIZE W
 		new_W = normalizeW(new_W);
	
	
		%%%%%%%%%SEE IF COST WAS REDUCED
		Lambda = lin_reconstruct(new_W,H,b);
		new_cost = calc_cost(V,Lambda,sparse_factor,H);
		
		if (new_cost < cost)
			W = new_W;
			nuW = nuW*1.2;
			fprintf('new_cost: %d\n',new_cost);
			cost = new_cost;
			break;
		else
			nuW = nuW/2;
			fprintf('.');
			if(nuW < 1e-100)
				fprintf('Stuck...\n');
				return
			end
		end
		
	end
	
	while (1==1)
		%%%%%%%%%UPDATE H
		dH = zeros(D,J,PHI);
		
		normH = (sum(sum(sum(H.^.5))));

		for phi = 1:PHI
			for tau = 1:TAU
				dH(:,:,phi) = dH(:,:,phi) + SF_v_linear_shift(W(:,:,tau),(phi-1),b)'*SF_h_shift(V-Lambda,-(tau-1));
			end
			aux1 = H(:,:,tau).^(-.5);
			new_H(:,:,phi) = H(:,:,phi) + nuH*(sparse_factor*aux1/normH + dH(:,:,phi));
		end
		
		new_H(new_H <= 0) = eps;
	
		%%%%%%%%%SEE IF COST WAS REDUCED
		Lambda = lin_reconstruct(W,new_H,b);
		new_cost = calc_cost(V,Lambda,sparse_factor,new_H);
		nuH
		if (new_cost < cost)
			H = new_H;
			nuH = nuH*1.2;
			fprintf('new_cost: %d\n',new_cost);
			cost = new_cost;
			break;
		else
			nuH = nuH/2;
			fprintf('.');
			if(nuH < 1e-100)
				fprintf('Stuck...\n');
				return
			end
		end
		
	end

	if(strcmp(disp_plot,'on'))
		SF_disp_plot_f(W,H,Lambda);
	end
	if(strcmp(disp_targ,'on'))
		fprintf('Div: %d\n',cost);
	end;

end %while

end %function

function cost = calc_cost(V,Lambda,sparse_factor,H)
	cost = .5*sum(sum((V - Lambda).^2)) + sparse_factor*(sum(sum(sum(H.^.5))))^2;
end

function W = normalizeW(W)
	Q(1,:,1) = sqrt(sum(sum(W.^2,1),3));
	W = W./repmat(Q+eps,[size(W,1),1,size(W,3)]);
end

function Lambda = lin_reconstruct(W,H,b)

D = size(W,2);
TAU = size(W,3);
PHI = size(H,3);

Lambda = zeros(size(W,1),size(H,2));	

for tau = 1:TAU
		for phi = 1:PHI
				Lambda = Lambda + SF_v_linear_shift(W(:,:,tau),phi-1,b)*SF_h_shift(H(:,:,phi),tau-1);
		end
end

end