function C = SF_log_frequency_mapping(fMin, fMax, Fs, WS, FFT_size ,b)

% C = SF_log_frequency_mapping(fMin, fMax, Fs, WS, FFT_size ,b)
%
% Builds a linear to log mapping matrix
% INPUT:
% fMin: Lower frequency
% fMax: Highestfrequency
% FS: Sample Rate
% WS: Window Size
% FFT_size: must be greater than WS
% b: log bins for each octave
%
% OUTPUT
% C: linear to log mapping matrix
%
% Author: Alan F. Tygel
% Last Modified: 02/11/2009



  t = 1;
  k = 1;
  f = fMin;

  n_log_bins = round(log2(fMax/fMin)*b);

  %number of logarithmic bins
  n_log_bins = round(log2(fMax/fMin)*b);

  %transformation matrix
  C = zeros(n_log_bins, FFT_size/2);

  for l_b = 1:FFT_size/2

		%central freq of lin bin
    f = lin_bin_to_freq(l_b,FFT_size,Fs);

    if(f >= fMin)&&(f <= fMax)

      f_top = lin_bin_to_freq(l_b+1,FFT_size,Fs); %freq. of upper bin
      f_bot = lin_bin_to_freq(l_b-1,FFT_size,Fs); %freq. of bottom bin

      f_topedge = (f_top+f)/2; %top edge of linear bin (freq) 
      f_botedge = (f_bot+f)/2; %bottom edge of linear bin (freq)
			band_size = f_topedge - f_botedge; %freq bandsize

      [log_b_topedge log_b_topedge_b log_b_topedge_t] = freq_to_log_bin(f_topedge,fMin,b); %top edge of linear bin (log bin)

      [log_b_botedge log_b_botedge_b log_b_botedge_t] = freq_to_log_bin(f_botedge,fMin,b); %bot edge of linear bin (log bin)

			%frequency of the geometric mean between bins top and bottom of the edge
			flog_b_fronteira_top = log_bin_to_freq(log_b_topedge_t-.5,fMin,b);
			flog_b_fronteira_bot = log_bin_to_freq(log_b_botedge_b+.5,fMin,b);

			%verifiyng if they are inside the band; else, take the first inside
			if(flog_b_fronteira_top < f_topedge)
				flog_b_fronteira_top = log_bin_to_freq(log_b_topedge_t+.5,fMin,b);
			end
			if(flog_b_fronteira_bot > f_botedge)
				flog_b_fronteira_bot = log_bin_to_freq(log_b_botedge_b-.5,fMin,b);
			end
			
			%log bin under top edge
			[xx log_b_topedge xx] = freq_to_log_bin(flog_b_fronteira_top,fMin,b);
			
			%log bin over bottom edge
			[xx xx log_b_botedge] = freq_to_log_bin(flog_b_fronteira_bot,fMin,b);
		
			%assign energies for each log bin
			for log_b = log_b_botedge:log_b_topedge
				if((log_b > 0) && (log_b <= n_log_bins))
					bot_edge = log_bin_to_freq(log_b-.5,fMin,b);
	
					if(bot_edge < f_botedge)
						bot_edge = f_botedge;
					end
	
					top_edge = log_bin_to_freq(log_b+.5,fMin,b);
					if(top_edge > f_topedge)
						top_edge = f_topedge;
					end

					log_band_size = top_edge - bot_edge;
					%assign proportional energy
					C(log_b,l_b) = C(log_b,l_b) + log_band_size/band_size;
				end
			end
    end
  end
end



function f = lin_bin_to_freq(bin,WS,Fs)
bin_width = Fs/WS;
f = bin*bin_width;
end

function [b1 b2] = freq_to_lin_bin(f,WS,Fs)
  b = f/Fs*WS;
  b1 = floor(b);
  b2 = ceil(b);
end


function f = log_bin_to_freq(bin,fMin,b)
  f = fMin*power(2,bin/b);
end


function [bin b1 b2] = freq_to_log_bin(f,fMin,b)
  bin = log2(f/fMin)*b;
  b1 = floor(bin);
  b2 = ceil(bin); 
end
