function [W H KLDiv] = SF_nmf2d(X, TAU, PHI, D, max_it,delta,W_ini,H_ini)

% [W H] = SF_nmf2d(X, TAU, PHI, D, max_it, delta)
% Performs Non-negative matrix factor 2-D Deconvolution on log-spectrogram
% X using Kullback-Leibler Divergence as target function
%
% INPUT
% X: log-spectrogram
% TAU: number of frames in each factor
% PHI: possible frequency translations
% D: number of instruments
% max_it: maximum number of iterations
% delta: difference between successives values of target function under which
% convergence will be considered achieved
% W_ini: inital value for matrix W
% H_ini: inital value for matrix H
%
%
% OUTPUT
% W: W matrix at the end of optimization
% H: H matrix at the end of optimization
%
%
% Author: Alan F. Tygel
% Last Modified: 28/01/2009

global disp_targ disp_plot

X = abs(X);
I = size(X,1);
J = size(X,2);


    
%initialization for KLDiv
KLDiv = zeros(1,max_it);
% KLDiv_min = 10e10;
it = 2;
KLDiv(1) = 10e10;
KLDiv(2) = 10e10;

%eliminate zeros that may unstable the algorithm
V = X + 10e-9;

if (W_ini == 0)
	W = rand(I,D,TAU);
else
	W = W_ini;
end

if (H_ini == 0)
	H = rand(D,J,PHI);
else
	H = H_ini;
end

while (1==1)
%tic
it = it+1;

if(it == max_it)
    fprintf('Maximum iterations reached!\n');
    break;
end


Lambda = zeros(I,J);
for tau = 1:TAU
    for phi = 1:PHI
        Lambda = Lambda + SF_v_shift(W(:,:,tau),phi-1)*SF_h_shift(H(:,:,phi),tau-1);
    end
end


div = V./Lambda;

%%%%%%%%%%%%%%%%%%%K.L.Divergence
KLDiv(it) = sum(sum(V(:,:).*log(div(:,:)) - V(:,:) + Lambda(:,:)));


if(strcmp(disp_targ,'on'))
	fprintf('Div: %d\t It: %d\n',KLDiv(it),it);
	if(isnan(KLDiv(it)))
		error('Error: KLDiv is not a number.');
	end
end

if(abs(KLDiv(it)-KLDiv(it-2)) < delta)
    fprintf('Convergiu! Iteracao %d\n',it);
    break;
end

if(KLDiv(it)-KLDiv(it-2) > 0) 
    fprintf('Objective Function growing at it. %d. Stopping optimization.\n',it);
    break;
end

% if(KLDiv(it) < KLDiv_min)
% 	W_min = W;
% 	H_min = H;
% 	KLDiv_min = KLDiv(it);
% 	it_min = it;
% end


%%%%%%%%%MATRIX UPDATES%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(mod(it,2)==0)
  if(strcmp(disp_targ,'on')), fprintf('Update W \n'), end;
  %%%%W UPADTE
  AUX1 = zeros(I,D);
  AUX2 = zeros(I,D);
  for tau = 1:TAU
    for phi = 1:PHI
      AUX1 = AUX1 + SF_v_shift(div,-(phi-1))*SF_h_shift(H(:,:,phi),tau-1)';
      AUX2 = AUX2 + ones(I,J)*SF_h_shift(H(:,:,phi),tau-1)';    
    end
    W(:,:,tau) = W(:,:,tau) .* (AUX1./AUX2);
  end

else
  %%%%H UPDATE
  if(strcmp(disp_targ,'on')), fprintf('Update H \n'), end; 

  AUX1 = zeros(D,J);
  AUX2 = zeros(D,J);
  for phi = 1:PHI
    for tau = 1:TAU
      AUX1 = AUX1 + SF_v_shift(W(:,:,tau),phi-1)'*SF_h_shift(div,-(tau-1));
      AUX2 = AUX2 + SF_v_shift(W(:,:,tau),phi-1)'*ones(I,J);    
    end
  H(:,:,phi) = H(:,:,phi) .* (AUX1./AUX2);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end %if(mod(it,2)==0)


%%%%%%%%%%%%Matrixes Display %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(strcmp(disp_plot,'on'))
  SF_disp_plot_f(W,H,Lambda);
end


end %while


end %function