function [W H MSEDiv] = SF_nmf2d_MSE(X, TAU, PHI, D, max_it, delta,W_ini,H_ini)

% [W H] = SF_nmf2d_MSE(X, TAU, PHI, D, max_it, delta)
% Performs Non-negative matrix factor 2-D Deconvolution on log-spectrogram
% X using Mean-Square Error as target function.
%
% INPUT
% X: log-spectrogram
% TAU: number of frames in each factor
% PHI: possible frequency translations
% D: number of instruments
% max_it: maximum number of iterations
% delta: difference between successives values of target function under which
% convergence will be considered achieved
% W_ini: inital value for matrix W
% H_ini: inital value for matrix H
%
% OUTPUT
% W: W matrix at the end of optimization
% H: H matrix at the end of optimization
%
%
% Author: Alan F. Tygel
% Last Modified: 28/01/2009


global disp_targ disp_plot

V = abs(X) + 10e-9;
V = V./max(max(V));

I =size(X,1);
J =size(X,2);

if (W_ini == 0)
	W = rand(I,D,TAU);
else
	W = W_ini;
end

if (H_ini == 0)
	H = rand(D,J,PHI);
else
	H = H_ini;
end

it = 2;


% MSEDiv_min = 100000;
MSEDiv = zeros(1,max_it); 

MSEDiv(1) = 10e10;
MSEDiv(2) = 10e10;

while (1==1)
%tic
it = it+1;

if(it == max_it)
    fprintf('Maximum iterations reached!\n');
    break;
end


Lambda = zeros(I,J);
for tau = 1:TAU
    for phi = 1:PHI
        Lambda = Lambda + SF_v_shift(W(:,:,tau),phi-1)*SF_h_shift(H(:,:,phi),tau-1);
    end
end

%%%%%%%%%%%%%%%%%%%M.S.E
MSEDiv(it) = sum(sum((V(:,:) - Lambda(:,:)).^2));

if(strcmp(disp_targ,'on'))
	fprintf('Div: %d\t It: %d\n',MSEDiv(it),it);
	if(isnan(MSEDiv(it))), 
		error('Error: MSEDiv is not a number.')
	end
end

if(abs(MSEDiv(it)-MSEDiv(it-2)) < delta)
    fprintf('Converged! Iteration %d\n',it);
    break;
end

if(MSEDiv(it)-MSEDiv(it-2) > 0) 
    fprintf('Objective Function growing at it. %d. Stopping optimization.\n',it);
    break;
end

% if(MSEDiv(it) < MSEDiv_min)
% 	W_min = W;
% 	H_min = H;
% 	MSEDiv_min = MSEDiv(it);
% 	it_min = it;
% end


%%%%%%%%%MATRIX UPDATES%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if(mod(it,2)==0)
  if(strcmp(disp_targ,'on')), fprintf('Update W \n'), end;
  %%%%W UPADTE

  for tau = 1:TAU
  AUX1 = zeros(I,D);
  AUX2 = zeros(I,D);
    for phi = 1:PHI
      AUX1 = AUX1 + SF_v_shift(V,-(phi-1))*SF_h_shift(H(:,:,phi),tau-1)';
      AUX2 = AUX2 + SF_v_shift(Lambda,-(phi-1))*SF_h_shift(H(:,:,phi),tau-1)';    
    end
    W(:,:,tau) = W(:,:,tau) .* (AUX1./AUX2);
  end

else

  %%%%H UPDATE
  if(strcmp(disp_targ,'on')), fprintf('Update H \n'), end;

  for phi = 1:PHI
    AUX1 = zeros(D,J);
    AUX2 = zeros(D,J);
    for tau = 1:TAU
      AUX1 = AUX1 + SF_v_shift(W(:,:,tau),phi-1)'*SF_h_shift(V,-(tau-1));
      AUX2 = AUX2 + SF_v_shift(W(:,:,tau),phi-1)'*SF_h_shift(Lambda,-(tau-1));    
    end
    H(:,:,phi) = H(:,:,phi) .* (AUX1./AUX2);
  end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end %if(mod(it,2)==0)


if(strcmp(disp_plot,'on'))
  SF_disp_plot_f(W,H,Lambda);
end


end %while

MSEDiv  = MSEDiv(1:find(MSEDiv==0,1)-1);

end %function